module CustomFielder
  module DeserializationHelper

    FALSE_VALUES = [false, 0, '0', 'f', 'F', 'false', 'FALSE', 'off', 'OFF']

    ##
    # Runs the the deserialization function for the passed type
    #
    # @param type  [String] the type to deserialize to
    # @param value [String] the value to be deserialized
    #
    # @return [Array|Integer|Float|Date|DateTime|Boolean|String]
    #
    def deserialize(type, value)
      begin
        send("#{type.underscore}", value)
      rescue
        # This should only occur when trying to access the value
        # before the value was validated (before saving)
        value
      end
    end

    ##
    # @return [Array]
    #
    def array(value)
      value.delete('[]"').split(', ').map do |val|
        case val
        when /^\d+$/
          val.to_i
        when /^\d+\.\d+$/
          val.to_f
        when /^\d{4}-\d{2}-\d{2}$/
          date(val)
        when /^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}-\d{2}:\d{2}$/
          date_time(val)
        when 'true'
          true
        when 'false'
          false
        when /".+"/
          val.delete('"')
        else
          val
        end
      end
    end

    ##
    # @return [Integer]
    #
    def fixnum(value)
      value.to_i
    end

    ##
    # @return [Float]
    #
    def float(value)
      value.to_f
    end

    ##
    # @return [Date]
    #
    def date(value)
      Date.parse(value)
    end

    ##
    # @return [DateTime]
    #
    def date_time(value)
      DateTime.parse(value)
    end

    ##
    # @return [Boolean]
    #
    def boolean(value)
      FALSE_VALUES.include?(value) ? false : true
    end

    ##
    # @return [String]
    #
    def string(value)
      value
    end

  end
end
