module CustomFielder
  module TypeCheckingHelper

    TRUTH_VALUES = [true,  1, '1', 't', 'T', 'true',  'TRUE',  'on',  'ON']
    FALSE_VALUES = [false, 0, '0', 'f', 'F', 'false', 'FALSE', 'off', 'OFF']

    ##
    # Runs the the type checking function for the passed type
    #
    # @param type  [String] the type to be checked
    # @param value [String] the value to be type checked
    #
    # @return [Boolean]
    #
    def is_correct_type?(type, value)
      send("is_#{type.underscore}?", value)
    end

    ##
    # @return [Boolean]
    #
    def is_array?(value)
      value =~ /^\[(.(, )?)+\]$/
    end

    ##
    # @return [Boolean]
    #
    def is_fixnum?(value)
      value =~ /^\d+$/
    end

    ##
    # @return [Boolean]
    #
    def is_float?(value)
      value =~ /^\d+\.\d+$/
    end

    ##
    # @return [Boolean]
    #
    def is_date?(value)
      value =~ /^\d{4}-\d{2}-\d{2}$/
    end

    ##
    # @return [Boolean]
    #
    def is_date_time?(value)
      value =~ /^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}$/ or
      value =~ /^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}$/ or
      value =~ /^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\d{3}$/ or
      value =~ /^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}-\d{2}:\d{2}$/
    end

    ##
    # @return [Boolean]
    #
    def is_boolean?(value)
      TRUTH_VALUES.include?(value) or FALSE_VALUES.include?(value)
    end

    ##
    # @return [Boolean]
    #
    def is_string?(value)
      true
    end

  end
end
